/*
Data in array is as follows
 0 - Left Shoulder L/R
 1 - Left Shoulder U/D
 2 - Left Elbow
 3 - Left Wrist
 4 - Left Wrist L/R
 5 - Left Hand

 6 - Right Shoulder L/R
 7 - Right Shoulder U/D
 8 - Right Elbow
 9 - Right Wrist
10 - Right Wrist L/R
11 - Right Hand

12 - Turret U/D
13 - Turret L/R

14 - Finger data
15 - US Range Data (1" units)
16 - IR Range Data (1" units)
17 - Limit (for scan)
18 - Request Scan  Bit 0-IR  Bit 1-ultrasonic   Bit2: 1-Vert, 0-Horz   
19 - Request Close Hand (1-left, 2-right)
20 - Request Look (look left-right-for object)  1-Left, 2 Right
21 - Request ForwardUntil (move forward until object found) 1-Left, 2-Right  
22 - Reserved for expansion
23 - Reserved for expansion
24 - Reserved for expansion



 placing a number in a servo position will cause that motor to move to
 that position
 
 IR, US, and Finger Data will ALWAYS BE UPDATED

 placing -1 in CloseLeft, or CloseRight will cause it to close till switch activates
    both In/Out arrays are updated accordingly as well as the Close variables
 placing -1 in any of the SCAN variables will cause that scan to be made
 (7 positions are scanned at Close, Medium, Long, or Horizontal Range.
 If EITHER sensor finds an object LESS THAN the LIMIT VALUE then that bit 
 will be ON in the returned data position - LSB is Robot's Right Side)
 placing -1 in QuickData will read turret sensors and Return immediately without moving servos
 
 When Master Saves array it should be ReNamed ProcessNow
 When Slave performs actions and updates array it should be ReNamed DataReady
 
 When Slave starts, it should ensure that the array is named DataReady
 */
 

Main:  // this is the SLAVE program, SpeechControlDemo.bas is the MASTER program
  gosub Init 
  while TRUE
    repeat
    until FileExists ("ProcessNow")
    gosub Process
    FileRename("ProcessNow","DataReady")
  wend
end
 
Init:
  SetCommPort 7
  SerOut Char(0xAA)
  Data P;  1337,1332,1350,1257,1350,1425,1400,1330,1400,1375,1375,1075,1630,1780
  Data P;  0,0,0,0,0,0,0,0,0,0,0  
  Data Last; 1337,1332,1350,1257,1350,1425,1400,1330,1400,1375,1375,1075,1630,1780 
  // Horizontal turret positions for R to L scan
  Data H;    704,1012,1320,1630,1902,2174,2448
  Data V;    896,1073,1250,1427,1604,1780,1950 
  // vertical turret postions
  ClosedLeft = 2200   // fully closed
  ClosedRight = 1725
  if FileExists("ProcessNow") then FileDelete("ProcessNow")
  mWrite P,"DataReady"
  LHLF = 4
  LHRF = 1
  LHacross = 8
  LHswitch = 16
  RHLF = 64
  RHRF = 2
  RHacross = 128
  RHswitch = 32
return
 
Process:
  mRead P,"ProcessNow"
  gosub MoveServos
  if P[18] then gosub Scan
  if P[19] = 1 then gosub CloseLeftHand
  if P[19] = 2 then gosub CloseRightHand   
  if P[20] then gosub LookLR
  if P[21] then gosub ForwardUntil 
  mWrite P,"ProcessNow"
return

CloseLeftHand:
  for ih = Last[5] to ClosedLeft step 25
    SerOut char(0x84),char(5+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f) 
    gosub ReadSensors
    if P[14]&LHswitch then break
  next
  for it=ih to ih-25 step -5
    SerOut char(0x84),char(5+2),char(4*(it)&0x7f),char((4*(it)>>7)&0x7f)
    gosub ReadSensors
    if not P[14]&LHswitch then break
  next
  Last[5]=it
  P[5] = it  
  P[19]=0
return

CloseRightHand:
  for ih = Last[11] to ClosedRight step 25
    SerOut char(0x84),char(11+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f) 
    gosub ReadSensors
    if P[14]&RHswitch then break
  next
  for it=ih to ih-25 step -5
    SerOut char(0x84),char(11+2),char(4*(it)&0x7f),char((4*(it)>>7)&0x7f)
    gosub ReadSensors
    if not P[14]&RHswitch then break
  next
  Last[11]=it
  P[11] = it 
  P[19]=0
return

Scan:
  LimVal = P[17]
  IR = P[18]&1
  USonic = P[18]&2 
  bit=1
  Answer = 0
  for ii=0 to 6
    if P[18]&4  // Vertical scan
      SerOut char(0x84),char(13+2),char(4*V[ii]&0x7f),char( (4*V[ii]>>7)&0x7f)
    else  // horizontal scan     
      SerOut char(0x84),char(12+2),char(4*H[ii]&0x7f),char( (4*H[ii]>>7)&0x7f)
    endif       
    // Read sensors and convert to inches and compare to LimVal
    gosub ReadSensors
    if (IR and (P[16]<LimVal))  or (USonic and (P[15]<LimVal))
      Answer+=bit     
    endif 
    bit = bit<<1 
  next
  P[17] = Answer
  if P[18]&4
    SerOut char(0x84),char(13+2),char(4*V[3]&0x7f),char( (4*V[3]>>7)&0x7f)
  else  // horizontal scan     
    SerOut char(0x84),char(12+2),char(4*H[3]&0x7f),char( (4*H[3]>>7)&0x7f)
  endif
  P[18]=0   
return

MoveServos:
  for i = 0 to 13
    if P[i]!=Last[i]
      SerOut char(0x84),char(i+2),char(4*P[i]&0x7f),char( (4*P[i]>>7)&0x7f)
      Last[i] = P[i]
    endif 
  next
  gosub ReadSensors
return

LookLR:
  gosub ReadSensors

return

ForwardUntil:
  gosub ReadSensors

return

ReadSensors:
  FingerTotal=255
  IRtotal = 0
  UStotal = 0
  for k=1 to 4
    SerOut char(0xA7),char(0) // start script 0
    for j=1 to 2
      repeat
      SerBytesIn 1,DataRead,NumberRead
      until DataRead = char(j)
    next
    SerBytesIn 3,DataRead,NumberRead
    for j=1 to 3
      // place in positions 14,15, and 16
      P[13+j] = ascii(substring(DataRead,j,1))
    next 
    UStotal+=P[15]
    IRtotal+=P[16]   
    FingerTotal=FingerTotal&P[14] // reduce false readings 
  next
  P[15]=UStotal/4
  //P[16]=IRtotal/4
  // convert to inches
  P[15] = round(.48*P[15]) //round(P[15]*.48)
  
  if P[15]>60 then P[15]=255   
  if P[16]>=86
     P[16]=round(33-.1765*P[16])
  elseif P[16]>63
    P[16]=round(44-.3*P[16])
  elseif P[16]>=45
    P[16]=round(63.4-.61*P[16])
  elseif P[16]>=28
    P[16]=round(101.5-1.47*P[16])
  else
    P[16]=255
  endif   
  P[14]=FingerTotal
  // shows data for clarity - may be removed     
  for j= 1 to 3
    xyString 100,100+j*20, "<",P[13+j],"    >"
  next
return
